VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "okHTTPClient"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private WithEvents HTTPSocket As okSocket
Attribute HTTPSocket.VB_VarHelpID = -1

Private m_blnInitilized As Boolean

Private Type HTTPRequest
    ID As Variant
    sSocket As Long
    URL As String
    Port As Long
    DataHeader As String
    Data As String          'includes header
    DataSent As Long
    RecvHeader As String
    RecvData As String      'includes header
End Type
Private Requests() As HTTPRequest

Public Event Status(ByVal varID As Variant, ByVal strMsg As String)
Public Event ReceiveProgress(ByVal varID As Variant, ByVal lngRecv As Long, ByVal lngTotal As Long)
Public Event Received(ByVal varID As Variant, ByVal strResponse As String, ByVal strHeader As String, ByVal strData As String)
Public Event Closed(ByVal varID As Variant)

Public Function Initilize(ByVal aSocket As okSocket) As Boolean
    If Not m_blnInitilized Then
        Set HTTPSocket = aSocket
    
        ReDim Requests(0)
    
        m_blnInitilized = True
    End If

    Initilize = m_blnInitilized
End Function
Public Sub Terminate()
    Erase Requests
    m_blnInitilized = False
End Sub

'- HTTP tasks
'
Public Function Request(ByVal varID As Variant, ByVal strURL As String, Optional ByVal lngPort As Long = 80) As Boolean
    Dim strDomain As String, strRequest As String, strHeader As String
    
    If m_blnInitilized Then
        If LCase$(Left$(strURL, 8)) = "https://" Then
            RaiseEvent Status(0, "sorry, https not supported..")
        Else
            If LCase$(Left$(strURL, 7)) <> "http://" Then strURL = "http://" & strURL
        
            strDomain = GetDomainFromURL(strURL)
            If strDomain = "" Then
                Request = False
            Else
                strRequest = Mid$(strURL, Len(strDomain) + 8)
                If strRequest = "" Then strRequest = "/"
                
                strHeader = "GET " & strRequest & " HTTP/1.1" & vbCrLf & _
                    "Range: bytes=0-" & vbCrLf & _
                    "Host: " & strDomain & vbCrLf & _
                    "Accept: */*" & vbCrLf & _
                    "User-Agent: " & App.Title & "/" & App.Major & "." & App.Minor & vbCrLf & _
                    "Connection: close" & vbCrLf & vbCrLf
                
                Request = RequestAdd(HTTPSocket.Connect(IPPROTO_TCP, lngPort, strDomain), varID, strURL, lngPort, strHeader)
            End If
        End If
    End If
End Function
'Public Function Post()
'    'todo
'End Function
'Public Function SendFile()
'    'todo
'End Function

'- socket events
'
Private Sub HTTPSocket_Connected(ByVal lngSocket As Long)
    Call SendData(lngSocket, "connected. sending..")
End Sub
Private Sub HTTPSocket_DataSent(ByVal lngSocket As Long, ByVal lngTotal As Long)
    Dim intIndex As Integer
    
    intIndex = RequestFind(lngSocket)
    If intIndex > 0 Then
        With Requests(intIndex)
            If lngTotal < Len(.Data) Then
                RaiseEvent Status(.ID, "sending.. " & Format(lngTotal / Len(.Data), "###%"))
            Else
                RaiseEvent Status(.ID, "sent. waiting..")
            End If
        
            If lngTotal > 0 Then .DataSent = lngTotal
        End With
    Else
        Debug.Print "|" & lngSocket & "|HTTP sent connection not found"
    End If
End Sub
Private Sub HTTPSocket_ReadyToSend(ByVal lngSocket As Long)
    Call SendData(lngSocket, "sending again..")
End Sub
Private Sub HTTPSocket_Received(ByVal lngSocket As Long, Data() As Byte, ByVal lngLength As Long)
    Dim intIndex As Integer, varID As Variant, strResponse As String, strHeader As String, strData As String
    
    intIndex = RequestFind(lngSocket)
    If intIndex > 0 Then
        With Requests(intIndex)
            varID = .ID
            strHeader = .RecvHeader
            strData = .RecvData
        End With
            
        If lngLength = 0 Then
            'get response code and message
            If UCase$(Left$(strHeader, 7)) = "HTTP/1." Then
                strResponse = Mid$(strHeader, 10, InStr(1, strHeader, vbCrLf) - 10)
            End If
            
            '1.1 clients must deal with chunked encoding
            If LCase$(GetHeaderValue(strHeader, "transfer-encoding")) = "chunked" Then
                RaiseEvent Received(varID, strResponse, strHeader, GetChunkedData(Mid$(strData, Len(strHeader) + 1)))
            Else
                RaiseEvent Received(varID, strResponse, strHeader, Mid$(strData, Len(strHeader) + 1))
            End If
        ElseIf lngLength > 0 Then
            strData = strData & Left$(StrConv(Data, vbUnicode), lngLength)
            
            If strHeader = "" Then
                strHeader = GetHeaderFromData(strData)
            Else
                RaiseEvent ReceiveProgress(varID, Len(strData) - Len(strHeader), CLng(Val(GetHeaderValue(strHeader, "content-length"))))
            End If
        End If
            
        With Requests(intIndex)
            .ID = varID
            .RecvHeader = strHeader
            .RecvData = strData
        End With
    Else
        Debug.Print "|" & lngSocket & "|HTTP received connection not found"
    End If
End Sub
Private Sub HTTPSocket_Closed(ByVal lngSocket As Long)
    Dim intIndex As Integer
    
    intIndex = RequestFind(lngSocket)
    If intIndex > 0 Then
        RaiseEvent Closed(Requests(intIndex).ID)
        Call RequestRemove(intIndex)
    End If
End Sub
Private Sub HTTPSocket_Error(ByVal lngSocket As Long, ByVal strMsg As String)
    Debug.Print "http socket error: " & strMsg
End Sub

'since some of the connected and readytosend code is the same, make a sub
Private Sub SendData(ByVal lngSocket As Long, ByVal strMsg As String)
    Dim intIndex As Integer, varID As Variant, strData As String, lngDataSent As Long
    
    intIndex = RequestFind(lngSocket)
    If intIndex > 0 Then
        varID = Requests(intIndex).ID
        strData = Requests(intIndex).Data
        lngDataSent = Requests(intIndex).DataSent
    
        If lngDataSent < Len(strData) Then
            RaiseEvent Status(varID, strMsg)
            
            Call HTTPSocket.Send(lngSocket, StrConv(strData, vbFromUnicode), lngDataSent)
        End If
    Else
        Debug.Print "|" & lngSocket & "|HTTP send connection not found"
    End If
End Sub

'- request array list type code
'
Private Function RequestAdd(ByVal lngSocket As Long, ByVal varID As Variant, ByVal strURL As String, ByVal lngPort As Long, ByVal strData As String) As Boolean
    Dim intNew As Integer
    
    If lngSocket > 0 Then
        intNew = UBound(Requests) + 1
        ReDim Preserve Requests(0 To intNew)
        
        With Requests(intNew)
            .ID = varID
            .sSocket = lngSocket
            .URL = strURL
            .Port = lngPort
            .DataHeader = GetHeaderFromData(strData)
            .Data = strData
            .DataSent = 0
            .RecvHeader = ""
            .RecvData = ""
        End With
        
        RequestAdd = True
    End If
End Function
Private Function RequestFind(ByVal lngSocket As Long) As Integer
    Dim i As Integer, intIndex As Integer
  
    For i = 1 To UBound(Requests)
        If Requests(i).sSocket = lngSocket Then
            intIndex = i
            Exit For
        End If
    Next
    
    RequestFind = intIndex
End Function
Private Sub RequestRemove(ByVal intIndex As Integer)
    Dim lngUBound As Long, i As Long
    
   On Error GoTo RequestRemove_Error

    If intIndex > 0 Then
        lngUBound = UBound(Requests)
        If lngUBound = 1 Then
            Erase Requests()
            ReDim Requests(0)
        ElseIf lngUBound >= 2 Then
            For i = intIndex + 1 To UBound(Requests)
                Requests(i - 1).ID = Requests(i).ID
                Requests(i - 1).sSocket = Requests(i).sSocket
                Requests(i - 1).URL = Requests(i).URL
                Requests(i - 1).Port = Requests(i).Port
                Requests(i - 1).DataHeader = Requests(i).DataHeader
                Requests(i - 1).Data = Requests(i).Data
                Requests(i - 1).DataSent = Requests(i).DataSent
                Requests(i - 1).RecvHeader = Requests(i).RecvHeader
                Requests(i - 1).RecvData = Requests(i).RecvData
            Next
            ReDim Preserve Requests(0 To UBound(Requests) - 1)
        End If
    End If

   On Error GoTo 0
   Exit Sub

RequestRemove_Error:

    MsgBox "Error " & Err.Number & " (" & Err.Description & ") in procedure RequestRemove of Class Module okHTTPClient"
End Sub

'- helpers
'
Private Function GetDomainFromURL(ByVal strURL As String) As String
    On Error Resume Next
    
    If LCase$(Left$(strURL, 7)) = "http://" Then strURL = Mid$(strURL, 8)
    GetDomainFromURL = Split(strURL, "/")(0)
End Function
Private Function GetHeaderFromData(ByVal strData As String) As String
    Dim lngStart As Long
    
    lngStart = InStr(1, strData, vbCrLf & vbCrLf)
    If lngStart > 0 Then GetHeaderFromData = Mid$(strData, 1, lngStart + 3)
End Function
Private Function GetHeaderValue(ByVal strHeader As String, ByVal strName As String) As String
    Dim aHead() As String, i As Integer
    
    aHead = Split(strHeader, vbCrLf)
    
    For i = 0 To UBound(aHead)
        If LCase$(Left$(aHead(i), Len(strName) + 2)) = LCase$(strName) & ": " Then
            GetHeaderValue = Trim$(Mid$(aHead(i), Len(strName) + 3))
        End If
    Next
End Function
Private Function GetChunkedData(ByVal strData As String) As String
    Dim lngCrLf As Long, lngChunk As Long, strOut As String
    
    On Error GoTo ChunkErr
    
    Do
        lngCrLf = InStr(1, strData, vbCrLf)
        If lngCrLf > 0 Then
            lngChunk = CLng("&H" & Split(Left$(strData, lngCrLf - 1), ";")(0))
            If lngChunk >= 0 Then
                strOut = strOut & Mid$(strData, lngCrLf + 2, lngChunk)
                strData = Mid$(strData, lngCrLf + lngChunk + 4)
            End If
        End If
    Loop Until lngCrLf <= 0
    
    GetChunkedData = strOut
ChunkErr:
    'returns empty string
End Function
