VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "okSocket"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'okSocket - a Windows Socket 2 implementation by Kris Kolody
'
' some socket info and code from winsockvb.com (http://web.archive.org/web/*/http://www.winsockvb.com),
' CSocketMaster/Plus (planetsourcecode.com) and MSDN (http://msdn2.microsoft.com/en-us/library/ms740673.aspx)
' uses customized subclassing code from Paul Caton's self-subclassing class (in cSelfSub.cls)

' Winsock version, who needs 1.1?
Private Const WINSOCK_V2_2  As Long = &H202

' Length of fields within the WSADATA structure.
Private Const WSADESCRIPTION_LEN  As Long = 256
Private Const WSASYS_STATUS_LEN   As Long = 128

' To initialize Winsock.
Private Type WSADATA
   wVersion                               As Integer
   wHighVersion                           As Integer
   szDescription(WSADESCRIPTION_LEN + 1)  As Byte
   szSystemstatus(WSASYS_STATUS_LEN + 1)  As Byte
   iMaxSockets                            As Integer
   iMaxUpdDg                              As Integer
   lpVendorInfo                           As Long
End Type

Private Const SOCKET_ERROR        As Long = -1

' Internet addresses.
Private Const INADDR_ANY          As Long = &H0
Private Const INADDR_NONE         As Long = &HFFFFFFFF

' Maximum backlog when calling listen().
Private Const SOMAXCONN As Long = 5

' Messages send with WSAAsyncSelect().
Private Const FD_READ       As Long = &H1
Private Const FD_WRITE      As Long = &H2
Private Const FD_ACCEPT     As Long = &H8
Private Const FD_CONNECT    As Long = &H10
Private Const FD_CLOSE      As Long = &H20

' GetWinsockError constants.
Private Const WSABASEERR          As Long = 10000
Private Const WSAEINTR            As Long = WSABASEERR + 4
Private Const WSAEACCES           As Long = WSABASEERR + 13
Private Const WSAEFAULT           As Long = WSABASEERR + 14
Private Const WSAEINVAL           As Long = WSABASEERR + 22
Private Const WSAEMFILE           As Long = WSABASEERR + 24
Private Const WSAEWOULDBLOCK      As Long = WSABASEERR + 35
Private Const WSAEINPROGRESS      As Long = WSABASEERR + 36
Private Const WSAEALREADY         As Long = WSABASEERR + 37
Private Const WSAENOTSOCK         As Long = WSABASEERR + 38
Private Const WSAEDESTADDRREQ     As Long = WSABASEERR + 39
Private Const WSAEMSGSIZE         As Long = WSABASEERR + 40
Private Const WSAEPROTOTYPE       As Long = WSABASEERR + 41
Private Const WSAENOPROTOOPT      As Long = WSABASEERR + 42
Private Const WSAEPROTONOSUPPORT  As Long = WSABASEERR + 43
Private Const WSAESOCKTNOSUPPORT  As Long = WSABASEERR + 44
Private Const WSAEOPNOTSUPP       As Long = WSABASEERR + 45
Private Const WSAEPFNOSUPPORT     As Long = WSABASEERR + 46
Private Const WSAEAFNOSUPPORT     As Long = WSABASEERR + 47
Private Const WSAEADDRINUSE       As Long = WSABASEERR + 48
Private Const WSAEADDRNOTAVAIL    As Long = WSABASEERR + 49
Private Const WSAENETDOWN         As Long = WSABASEERR + 50
Private Const WSAENETUNREACH      As Long = WSABASEERR + 51
Private Const WSAENETRESET        As Long = WSABASEERR + 52
Private Const WSAECONNABORTED     As Long = WSABASEERR + 53
Private Const WSAECONNRESET       As Long = WSABASEERR + 54
Private Const WSAENOBUFS          As Long = WSABASEERR + 55
Private Const WSAEISCONN          As Long = WSABASEERR + 56
Private Const WSAENOTCONN         As Long = WSABASEERR + 57
Private Const WSAESHUTDOWN        As Long = WSABASEERR + 58
Private Const WSAETOOMANYREFS     As Long = WSABASEERR + 59
Private Const WSAETIMEDOUT        As Long = WSABASEERR + 60
Private Const WSAECONNREFUSED     As Long = WSABASEERR + 61
Private Const WSAENAMETOOLONG     As Long = WSABASEERR + 63
Private Const WSAEHOSTDOWN        As Long = WSABASEERR + 64
Private Const WSAEHOSTUNREACH     As Long = WSABASEERR + 65
Private Const WSAENOTEMPTY        As Long = WSABASEERR + 66
Private Const WSAEPROCLIM         As Long = WSABASEERR + 67
Private Const WSASYSNOTREADY      As Long = WSABASEERR + 91
Private Const WSAVERNOTSUPPORTED  As Long = WSABASEERR + 92
Private Const WSANOTINITIALISED   As Long = WSABASEERR + 93
Private Const WSAEDISCON          As Long = WSABASEERR + 101
Private Const WSATYPE_NOT_FOUND   As Long = WSABASEERR + 109
Private Const WSAHOST_NOT_FOUND   As Long = WSABASEERR + 1001
Private Const WSATRY_AGAIN        As Long = WSABASEERR + 1002
Private Const WSANO_RECOVERY      As Long = WSABASEERR + 1003
Private Const WSANO_DATA          As Long = WSABASEERR + 1004

' Basic IPv4 addressing structures.
Private Type in_addr
   s_addr   As Long
End Type

Private Type sockaddr_in
   sin_family        As Integer
   sin_port          As Integer
   sin_addr          As in_addr
   sin_zero(0 To 7)  As Byte
End Type

' Used with name resolution functions.
Private Type hostent
   h_name         As Long
   h_aliases      As Long
   h_addrtype     As Integer
   h_length       As Integer
   h_addr_list    As Long
End Type

' Used with socket().
Private Enum SocketTypes
   SOCK_STREAM = 1
   SOCK_DGRAM = 2
End Enum

' Used with socket().
Public Enum Protocols
   IPPROTO_TCP = 6
   IPPROTO_UDP = 17
End Enum

' Used with socket().
Private Enum AddressFamilies
   AF_INET = 2
End Enum

' DLL handling functions.
Private Declare Function WSAStartup Lib "ws2_32.dll" (ByVal wVersionRequested As Integer, ByRef lpWSAData As WSADATA) As Long
Private Declare Function WSACleanup Lib "ws2_32.dll" () As Long
Private Declare Function WSAGetLastError Lib "ws2_32.dll" () As Long

' Resolution functions.
Private Declare Function gethostbyname Lib "ws2_32.dll" (ByVal host_name As String) As Long
Private Declare Function gethostbyaddr Lib "ws2_32.dll" (haddr As Long, ByVal hnlen As Long, ByVal addrtype As Long) As Long

' Conversion functions.
Private Declare Function inet_addr Lib "ws2_32.dll" (ByVal cp As String) As Long
Private Declare Function inet_ntoa Lib "ws2_32.dll" (ByVal laddr As Long) As Long
Private Declare Function htons Lib "ws2_32.dll" (ByVal hostshort As Long) As Integer

' Socket functions.
Private Declare Function ws2_socket Lib "ws2_32.dll" Alias "socket" (ByVal af As AddressFamilies, ByVal stype As SocketTypes, ByVal lngProtocol As Long) As Long
Private Declare Function ws2_bind Lib "ws2_32.dll" Alias "bind" (ByVal s As Long, ByRef Name As sockaddr_in, ByVal namelen As Long) As Long
Private Declare Function ws2_listen Lib "ws2_32.dll" Alias "listen" (ByVal s As Long, ByVal backlog As Long) As Long
Private Declare Function ws2_accept Lib "ws2_32.dll" Alias "accept" (ByVal s As Long, ByRef addr As sockaddr_in, ByRef addrlen As Long) As Long
Private Declare Function ws2_connect Lib "ws2_32.dll" Alias "connect" (ByVal s As Long, ByRef Name As sockaddr_in, ByVal namelen As Long) As Long
Private Declare Function ws2_sendto Lib "ws2_32.dll" Alias "sendto" (ByVal s As Long, ByRef buf As Byte, ByVal datalen As Long, ByVal Flags As Long, ByRef toaddr As sockaddr_in, ByVal tolen As Long) As Long
Private Declare Function ws2_recv Lib "ws2_32.dll" Alias "recv" (ByVal s As Long, ByRef buf As Byte, ByVal datalen As Long, ByVal Flags As Long) As Long
Private Declare Function ws2_closesocket Lib "ws2_32.dll" Alias "closesocket" (ByVal s As Long) As Long

' I/O model functions.
Private Declare Function WSAAsyncSelect Lib "ws2_32.dll" (ByVal s As Long, ByVal hWnd As Long, ByVal wMsg As Long, ByVal lEvent As Long) As Long

' Other general Win32 APIs.
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (pDst As Any, pSrc As Any, ByVal ByteLen As Long)
Private Declare Function RegisterWindowMessage Lib "user32" Alias "RegisterWindowMessageA" (ByVal lpString As String) As Long

Private m_lngHwnd As Long
Private m_lngSocketMsg As Long
Private m_blnInitilized As Boolean

Private Type Connection
    sSocket As Long
    SocketAddress As sockaddr_in
    RecvRaised As Boolean
End Type
Private ConnList() As Connection

Public Event Connected(ByVal lngSocket As Long)
Public Event Accepted(ByVal lngSocket As Long, ByVal strAddr As String, ByVal intPort As Integer)
Public Event Received(ByVal lngSocket As Long, Data() As Byte, ByVal lngLength As Long)
Public Event DataSent(ByVal lngSocket As Long, ByVal lngTotal As Long)
Public Event ReadyToSend(ByVal lngSocket As Long)
Public Event Closed(ByVal lngSocket As Long)
Public Event Error(ByVal lngSocket As Long, ByVal strMsg As String)

Public Function Initilize(hWnd As Long, cSubclass As cSelfSub) As Boolean
    Dim udtWinsockData As WSADATA
    
    If Not m_blnInitilized Then
        If WSAStartup(WINSOCK_V2_2, udtWinsockData) <> SOCKET_ERROR Then
            Randomize
            m_lngSocketMsg = RegisterWindowMessage("okSocket" & Rnd * 10000 & hWnd)
            m_lngHwnd = hWnd
            m_blnInitilized = True
            
            ReDim ConnList(0)
        
            'subclass the window to get socket messages
            Call cSubclass.SubclassWindow(m_lngHwnd, Me)
            Call cSubclass.SubclassMessage(m_lngHwnd, m_lngSocketMsg)
        End If
    End If
    
    Initilize = m_blnInitilized
End Function
Public Sub Terminate()
    Dim i As Long
  
    On Error Resume Next
    
    If m_blnInitilized Then 'close all sockets
        m_lngSocketMsg = 0
        m_lngHwnd = 0
        m_blnInitilized = False
        
        For i = 1 To UBound(ConnList)
            Call CloseSocket(ConnList(i).sSocket)
        Next i
        
        ReDim ConnList(0)
        
        Call WSACleanup
    End If
End Sub

Private Sub Class_Terminate()
    Call Terminate
End Sub

'you can connect or listen
Public Function Connect(ByVal Protocol As Protocols, ByVal Port As Long, ByVal Host As String) As Long
    Dim lngSocket As Long, lngInetAddr As Long, udtAddress As sockaddr_in
    
    lngSocket = ws2_socket(AF_INET, IIf(Protocol = IPPROTO_TCP, SOCK_STREAM, SOCK_DGRAM), Protocol)
    
    If SocketSelect(lngSocket, FD_READ Or FD_WRITE Or FD_CONNECT Or FD_CLOSE) Then
        lngInetAddr = inet_addr(vbIPFromHostName(Host))
        If lngInetAddr = INADDR_NONE Then
            Call CloseSocket(lngSocket)
            RaiseEvent Error(lngSocket, "connect error: server not found")
        Else
            With udtAddress
                .sin_family = AF_INET
                .sin_addr.s_addr = lngInetAddr
                .sin_port = htons(Port)
            End With
            Call ConnAdd(lngSocket, udtAddress)
            
            Call ws2_connect(lngSocket, udtAddress, LenB(udtAddress))
            
            Connect = lngSocket
        End If
    End If
End Function
Public Function Listen(ByVal Protocol As Protocols, ByVal Port As Long, Optional ByVal IP As String = INADDR_ANY) As Long
    Dim lngSocket As Long, udtAddress As sockaddr_in
    
    lngSocket = ws2_socket(AF_INET, IIf(Protocol = IPPROTO_TCP, SOCK_STREAM, SOCK_DGRAM), Protocol)
    
    If SocketSelect(lngSocket, FD_READ Or FD_WRITE Or FD_ACCEPT Or FD_CLOSE) Then
        With udtAddress
            .sin_family = AF_INET
            .sin_addr.s_addr = inet_addr(IP)
            .sin_port = htons(Port)
        End With
        Call ConnAdd(lngSocket, udtAddress)
        
        If ws2_bind(lngSocket, udtAddress, LenB(udtAddress)) = 0 Then
            Call ws2_listen(lngSocket, SOMAXCONN)
            
            Listen = lngSocket
        Else
            'could not bind
            Call CloseSocket(lngSocket)
        End If
    End If
End Function

'while listening, called when someone is trying to connect
Private Sub SocketAccept(ByVal lngSocket As Long)
    Dim lngAcceptSocket As Long, udtAddress As sockaddr_in
    
    lngAcceptSocket = ws2_accept(lngSocket, udtAddress, LenB(udtAddress))
    If SocketSelect(lngAcceptSocket, FD_READ Or FD_WRITE Or FD_CLOSE) Then
        Call ConnAdd(lngAcceptSocket, udtAddress)
        
        RaiseEvent Accepted(lngAcceptSocket, vbInetNtoa(udtAddress.sin_addr.s_addr), udtAddress.sin_port)
    End If
End Sub

'used by connect, listen and accept to get messages from the given socket
Private Function SocketSelect(ByVal lngSocket As Long, ByVal Events As Long) As Boolean
    If lngSocket = SOCKET_ERROR Then
        RaiseEvent Error(lngSocket, "create socket error: " & GetWinsockError(WSAGetLastError))
    Else
        If WSAAsyncSelect(lngSocket, m_lngHwnd, m_lngSocketMsg, Events) <> SOCKET_ERROR Then
            SocketSelect = True
        Else
            RaiseEvent Error(lngSocket, "wsaasyncselect error: " & GetWinsockError(WSAGetLastError))
            Call CloseSocket(lngSocket)
        End If
    End If
End Function

'called when a socket received data
Private Sub Recv(ByVal lngSocket As Long)
    Dim intConn As Integer, bytRecvBuffer() As Byte, lngRecv As Long, lngError As Long
    Const BUFFER_SIZE As Long = 4096
    
    On Error GoTo RecvErr
    
    intConn = ConnFind(lngSocket)
    
    'receive all data into a buffer
    Do
        Erase bytRecvBuffer
        ReDim bytRecvBuffer(BUFFER_SIZE - 1)
        
        lngRecv = ws2_recv(lngSocket, bytRecvBuffer(0), BUFFER_SIZE, 0&)
        If lngRecv = SOCKET_ERROR Then
            lngError = WSAGetLastError
            If lngError <> WSAEWOULDBLOCK And lngError <> 0 Then
                RaiseEvent Error(lngSocket, "receive error: " & GetWinsockError(lngError))
                'Call CloseSocket(lngSocket)
            End If
        Else
            'raise if connection found
            If intConn > 0 Then
                'Debug.Print "|" & CStr(lngSocket) & "|recv: " & CStr(lngRecv) & " bytes"
                If lngRecv = 0 Then
                    If Not ConnList(intConn).RecvRaised Then
                        ConnList(intConn).RecvRaised = True
                        ReDim bytRecvBuffer(0)
                        RaiseEvent Received(lngSocket, bytRecvBuffer, lngRecv)
                    End If
                Else
                    If lngRecv < BUFFER_SIZE Then ReDim Preserve bytRecvBuffer(0 To lngRecv - 1)
                    RaiseEvent Received(lngSocket, bytRecvBuffer, lngRecv)
                End If
            Else
                Debug.Print "|" & CStr(lngSocket) & "|recv: connection not found"
            End If
        End If
    Loop Until lngRecv <= 0
    
    Exit Sub
RecvErr:
    Debug.Print Err.Number & ": " & Err.Description
    Call CloseSocket(lngSocket)
End Sub

Public Sub Send(ByVal lngSocket As Long, Data() As Byte, ByVal lngStart As Long)
    Dim intConn As Integer, lngSend As Long, bytSendBuffer() As Byte, lngSize As Long, lngError As Long
    
    'On Error GoTo SendErr
    
    intConn = ConnFind(lngSocket)
    If intConn = 0 Then
        'not found, error then close
        Debug.Print "|" & CStr(lngSocket) & "|send: connection not found"
        Call CloseSocket(lngSocket)
    Else
        'copy from data into buffer
        Do
            lngSize = (UBound(Data) + 1) - lngStart
            If lngSize > 8192 Then lngSize = 8192
        
            If lngSize <= 0 Then
                'Debug.Print "|" & CStr(lngSocket) & "|send: no more bytes to send"
                lngSend = 0
            Else
                Erase bytSendBuffer
                ReDim bytSendBuffer(lngSize - 1)
                Call CopyMemory(bytSendBuffer(0), Data(lngStart), lngSize)
            
                lngSend = ws2_sendto(lngSocket, bytSendBuffer(0), lngSize, 0, ConnList(intConn).SocketAddress, LenB(ConnList(intConn).SocketAddress))
                If lngSend = SOCKET_ERROR Then
                    'error, if not blocking close socket
                    lngError = WSAGetLastError
                    If lngError = WSAEWOULDBLOCK Then
                        RaiseEvent DataSent(lngSocket, -1)      'raise with -1
                    Else
                        RaiseEvent Error(lngSocket, "send error: " & GetWinsockError(lngError))
                        Call CloseSocket(lngSocket)
                    End If
                Else
                    'Debug.Print "|" & CStr(lngSocket) & "|send: " & CStr(lngSend) & " bytes"
                    lngStart = lngStart + lngSend
                    RaiseEvent DataSent(lngSocket, lngStart)
                End If
            End If
        Loop Until lngSend <= 0
    End If
    
    Exit Sub
SendErr:
    Debug.Print Err.Number & ": " & Err.Description
    Call CloseSocket(lngSocket)
End Sub

Public Sub CloseSocket(ByVal lngSocket As Long)
    Call ws2_closesocket(lngSocket)
    Call ConnRemove(ConnFind(lngSocket))
    
    RaiseEvent Closed(lngSocket)
End Sub

'- Connections List Code
'
Private Function ConnAdd(ByVal lngSocket As Long, Address As sockaddr_in) As Integer
On Error GoTo ConnAddErr
    Dim intNew As Integer
    
    intNew = UBound(ConnList) + 1
    
    ReDim Preserve ConnList(0 To intNew)
    ConnList(intNew).sSocket = lngSocket
    ConnList(intNew).SocketAddress = Address
    
    ConnAdd = intNew
    
    Exit Function
ConnAddErr:
    Debug.Print "ConnAdd() Error: " & Err.Number & ": " & Err.Description
End Function
Private Function ConnFind(ByVal lngSocket As Long) As Integer
    Dim i As Integer, intIndex As Integer

    For i = 1 To UBound(ConnList)
        If ConnList(i).sSocket = lngSocket Then
            intIndex = i
            Exit For
        End If
    Next
    
    ConnFind = intIndex
End Function
Private Sub ConnRemove(ByVal intIndex As Integer)
On Error GoTo ConnRemoveErr
    Dim lngUBound As Long, i As Long
    
    If intIndex > 0 Then
        lngUBound = UBound(ConnList)
        If lngUBound = 1 Then
            ReDim ConnList(0)
        ElseIf lngUBound >= 2 Then
            For i = intIndex + 1 To UBound(ConnList)
                ConnList(i - 1).sSocket = ConnList(i).sSocket
                ConnList(i - 1).SocketAddress = ConnList(i).SocketAddress
                ConnList(i - 1).RecvRaised = ConnList(i).RecvRaised
            Next
            ReDim Preserve ConnList(0 To UBound(ConnList) - 1)
        End If
    End If
    
    Exit Sub
ConnRemoveErr:
    Debug.Print "ConnRemove() Error: " & intIndex
End Sub

Private Function vbInetNtoa(ByVal lngIPAddress As Long) As String
    Dim lngPointer As Long, strBuffer As String
    
    lngPointer = inet_ntoa(lngIPAddress)
    If lngPointer Then
        'prepare a buffer then copy the IP into it
        strBuffer = String$(16, 0)
        Call CopyMemory(ByVal strBuffer, ByVal lngPointer, Len(strBuffer))
        
        vbInetNtoa = Mid$(strBuffer, 1, InStr(1, strBuffer, vbNullChar) - 1)
    End If
End Function

Private Function vbHostNameFromIP(ByVal strIPAddress As String) As String
    Dim lngIPAddress As Long, udtHost As hostent, lngPointer As Long, strBuffer As String

    'convert the string IP to a long IP
    lngIPAddress = inet_addr(strIPAddress)
    If lngIPAddress <> INADDR_NONE Then

        'retrieve the hostent structure.
        lngPointer = gethostbyaddr(lngIPAddress, 4, AF_INET)
        If lngPointer Then
            'get the hostent structure from the pointer
            Call CopyMemory(udtHost, ByVal lngPointer, LenB(udtHost))
    
            'prepare a string buffer then copy the hostname from the h_name pointer
            strBuffer = String$(1024, 0)
            Call CopyMemory(ByVal strBuffer, ByVal udtHost.h_name, Len(strBuffer))
            
            vbHostNameFromIP = Mid$(strBuffer, 1, InStr(1, strBuffer, vbNullChar) - 1)
        End If
    End If
End Function
Private Function vbIPFromHostName(ByVal strHostName As String) As String
    Dim lngPointer As Long, udtHost As hostent, bytIP(0 To 3) As Byte
   
    'check if the hostname is already an IP
    If inet_addr(strHostName) <> INADDR_NONE Then
        vbIPFromHostName = strHostName
    Else
        lngPointer = gethostbyname(strHostName)
        If lngPointer Then
            'get the hostent structure from the pointer
            Call CopyMemory(udtHost, ByVal lngPointer, LenB(udtHost))
        
            'get the pointer needed from h_addr_list and h_length
            Call CopyMemory(lngPointer, ByVal udtHost.h_addr_list, udtHost.h_length)
        
            'get the IP address into a byte array from the pointer above
            Call CopyMemory(bytIP(0), ByVal lngPointer, udtHost.h_length)
        
            'return the IP address from the array
            vbIPFromHostName = bytIP(0) & "." & bytIP(1) & "." & bytIP(2) & "." & bytIP(3)
        End If
    End If
End Function

'error strings
Private Function GetWinsockError(ByVal lngErrorCode As Long) As String
    Select Case lngErrorCode
        Case WSAEINTR: GetWinsockError = "interrupted function call"
        Case WSAEACCES: GetWinsockError = "permission denied"
        Case WSAEFAULT: GetWinsockError = "invalid address"
        Case WSAEINVAL: GetWinsockError = "invalid argument"
        Case WSAEMFILE: GetWinsockError = "too many files open"
        Case WSAEWOULDBLOCK: GetWinsockError = "function call would block"
        Case WSAEINPROGRESS: GetWinsockError = "blocking call already in progress"
        Case WSAEALREADY: GetWinsockError = "operation already in progress"
        Case WSAENOTSOCK: GetWinsockError = "not a valid socket descriptor"
        Case WSAEDESTADDRREQ: GetWinsockError = "destination address required"
        Case WSAEMSGSIZE: GetWinsockError = "message is too long"
        Case WSAEPROTOTYPE: GetWinsockError = "protocol wrong type for socket"
        Case WSAENOPROTOOPT: GetWinsockError = "bad protocol option"
        Case WSAEPROTONOSUPPORT: GetWinsockError = "protocol not supported"
        Case WSAESOCKTNOSUPPORT: GetWinsockError = "socket type not supported"
        Case WSAEOPNOTSUPP: GetWinsockError = "operation not supported"
        Case WSAEPFNOSUPPORT: GetWinsockError = "protocol family not supported"
        Case WSAEAFNOSUPPORT: GetWinsockError = "address family not supported by protocol"
        Case WSAEADDRINUSE: GetWinsockError = "address in use"
        Case WSAEADDRNOTAVAIL: GetWinsockError = "address is not available"
        Case WSAENETDOWN: GetWinsockError = "network is down"
        Case WSAENETUNREACH: GetWinsockError = "network is unreachable"
        Case WSAENETRESET: GetWinsockError = "network dropped connection on reset"
        Case WSAECONNABORTED: GetWinsockError = "software caused connection abort"
        Case WSAECONNRESET: GetWinsockError = "connection reset by peer"
        Case WSAENOBUFS: GetWinsockError = "no buffer space available"
        Case WSAEISCONN: GetWinsockError = "socket is already connected"
        Case WSAENOTCONN: GetWinsockError = "socket is not connected"
        Case WSAESHUTDOWN: GetWinsockError = "cannot send after shutdown"
        Case WSAETOOMANYREFS: GetWinsockError = "too many socket references"
        Case WSAETIMEDOUT: GetWinsockError = "request timed out"
        Case WSAECONNREFUSED: GetWinsockError = "connection refused"
        Case WSAENAMETOOLONG: GetWinsockError = "name is too long"
        Case WSAEHOSTDOWN: GetWinsockError = "host is down"
        Case WSAEHOSTUNREACH: GetWinsockError = "host is unreachable"
        Case WSAEPROCLIM: GetWinsockError = "too many processes"
        Case WSASYSNOTREADY: GetWinsockError = "network sub-system is unavailable"
        Case WSAVERNOTSUPPORTED: GetWinsockError = "requested version not supported"
        Case WSANOTINITIALISED: GetWinsockError = "winsock is not loaded - call WSAStartup"
        Case WSAEDISCON: GetWinsockError = "graceful shutdown in progress"
        Case WSATYPE_NOT_FOUND: GetWinsockError = "class type not found"
        Case WSAHOST_NOT_FOUND: GetWinsockError = "host not found"
        Case WSATRY_AGAIN: GetWinsockError = "non-authoritative host not found"
        Case WSANO_RECOVERY: GetWinsockError = "non-recoverable error"
        Case WSANO_DATA: GetWinsockError = "valid name - no data record of requested type"
        Case Else: GetWinsockError = "unknown error(" + CStr(lngErrorCode) & ")"
    End Select
End Function

Private Function LoWord(ByVal lDWord As Long) As Integer
    If lDWord And &H8000& Then
        LoWord = lDWord Or &HFFFF0000
    Else
        LoWord = lDWord And &HFFFF&
    End If
End Function

'message handler, MUST be the last sub/function!!
Private Sub zWndProc1(ByVal bBefore As Boolean, ByRef bHandled As Boolean, ByRef lReturn As Long, _
                      ByVal lng_hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long, ByRef lParamUser As Long)
    
    If uMsg = m_lngSocketMsg Then
        Select Case LoWord(lParam)
            Case FD_READ
                Call Recv(wParam)
            Case FD_WRITE
                RaiseEvent ReadyToSend(wParam)
            Case FD_ACCEPT
                Call SocketAccept(wParam)
            Case FD_CONNECT
                RaiseEvent Connected(wParam)
            Case FD_CLOSE
                Call CloseSocket(wParam)
        End Select
    End If
End Sub
